﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis.Push.Data.Tree {
  using System.Linq;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;

  public static class TreeExtensions {
    public static Tree<T> ToTree<T>(
      this IEnumerable<T> items,
      Predicate<T> isChild,
      Func<T, IReadOnlyList<T>> childrenResolver) {
      var tree = new Tree<T>();

      foreach (var item in items) {
        if (isChild(item)) {
          var subNode = childrenResolver(item).ToTree(isChild, childrenResolver);
          subNode.Value = item;
          tree.AddNode(subNode);
        } else {
          tree.Add(item);
        }
      }

      return tree;
    }

    public static IEnumerable<TreeNode<T>> DepthFirst<T>(this TreeNode<T> node) {
      foreach (var child in node.Children) {
        if (child.Children.Count > 0) {
          foreach (var subChild in child.DepthFirst()) {
            yield return subChild;
          }
        } else yield return child;
      }
    }

    public static IEnumerable<T> DepthFirst<T>(this TreeNode<T> node, Func<IEnumerable<T>, T> resolveParent) {
      foreach (var child in node.Children) {
        if (child.Children.Count > 0) {
          var subExpressions = child.DepthFirst(resolveParent);
          yield return resolveParent(subExpressions);
        } else yield return child.Value;
      }
    }

    public static PushProgram ToPushProgram(this TreeNode<Expression> tree, Func<Expression, bool> condition = null) {
      var expressions = tree.DepthFirst(e => ResolveProgram(e, condition));

      return ResolveProgram(expressions, condition);
    }

    public static Tree<Expression> ToTree(this PushProgram program) {
      var tree = program.Expressions.ToTree(e => e.IsProgram, e => ((PushProgram)e).Expressions);
      tree.Value = program;

      return tree;
    }

    private static PushProgram ResolveProgram(IEnumerable<Expression> expressions, Func<Expression, bool> condition = null) {

      if (condition != null) {
        expressions = expressions.Where(condition);
      }

      return new PushProgram(expressions.ToArray());
    }
  }

}
