﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;

  using Stack;

  /// <summary>
  /// Tells whether that stack is empty.
  /// </summary>
  public abstract class EmptyExpresssions<T> : StatelessExpression {
    public bool Eval(IStack<T> stack, IStack<bool> booleanStack) {
      booleanStack.Push(stack.IsEmpty);

      return true;
    }
  }

  [PushExpression(StackTypes.Exec, "EXEC.EMPTY", StackTypes.Boolean)]
  public class ExecEmptyExpression : EmptyExpresssions<Expression> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.ExecStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Code, "CODE.EMPTY", StackTypes.Boolean)]
  public class CodeEmptyExpression : EmptyExpresssions<Expression> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.CodeStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Integer, "INTEGER.EMPTY", StackTypes.Boolean)]
  public class IntegerEmptyExpression : EmptyExpresssions<long> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.IntegerStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Float, "FLOAT.EMPTY", StackTypes.Boolean)]
  public class FloatEmptyExpression : EmptyExpresssions<double> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.FloatStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Boolean, "BOOLEAN.EMPTY", StackTypes.Boolean)]
  public class BooleanEmptyExpression : EmptyExpresssions<bool> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.BooleanStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.Char, "CHAR.EMPTY", StackTypes.Boolean)]
  public class CharEmptyExpression : EmptyExpresssions<char> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.CharStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.String, "STRING.EMPTY", StackTypes.Boolean)]
  public class StringEmptyExpression : EmptyExpresssions<string> {
    public override bool Eval(IPushInterpreter interpreter) {
      return Eval(interpreter.StringStack, interpreter.BooleanStack);
    }
  }
}
