﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Generators;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  ///     Pushes a random NAME.
  /// </summary>
  [PushExpression(StackTypes.Name, "NAME.RAND")]
  public class NameRandExpression : StatelessExpression {
    public override bool Eval(IPushInterpreter interpreter) {
      var name = NameGenerator.RandomName(interpreter.Random);

      interpreter.NameStack.Push(name);
      return true;
    }
  }

  /// <summary>
  ///     Pushes a random integer.
  /// </summary>
  [PushExpression(StackTypes.Integer, "INTEGER.RAND")]
  public class IntegerRandExpression : StatelessExpression {
    public override bool Eval(IPushInterpreter interpreter) {
      var value = interpreter.Random.Next(
          interpreter.Configuration.MinRandomInteger,
          interpreter.Configuration.MaxRandomInteger);

      interpreter.IntegerStack.Push(value);
      return true;
    }
  }

  /// <summary>
  ///     Pushes a random float.
  /// </summary>
  [PushExpression(StackTypes.Float, "FLOAT.RAND")]
  public class FloatRandExpression : StatelessExpression {
    public override bool Eval(IPushInterpreter interpreter) {
      var value = interpreter.Random.NextDouble() * (interpreter.Configuration.MaxRandomFloat - interpreter.Configuration.MinRandomFloat) + interpreter.Configuration.MinRandomFloat;

      interpreter.FloatStack.Push(value);
      return true;
    }
  }

  /// <summary>
  ///     Pushes a random boolean.
  /// </summary>
  [PushExpression(StackTypes.Boolean, "BOOLEAN.RAND")]
  public class BooleanRandExpression : StatelessExpression {
    public override bool Eval(IPushInterpreter interpreter) {
      var value = interpreter.Random.NextDouble() > 0.5;

      interpreter.BooleanStack.Push(value);
      return true;
    }
  }

  /// <summary>
  ///     Pushes random expressions onto the code stack.
  /// </summary>
  [PushExpression(StackTypes.Code, "CODE.RAND")]
  public class CodeRandExpression : StatelessExpression {
    public override bool Eval(IPushInterpreter interpreter) {
      if (interpreter.IntegerStack.Count == 0 ||
          interpreter.IntegerStack.Top < 1) return false;

      var size = (int)(interpreter.IntegerStack.Pop() % interpreter.Configuration.MaxPointsInRandomExpression);
      var program = LinearCodeGenerator.RandomProgram(
        interpreter.PoolContainer.PushProgramPool,
        interpreter.PoolContainer.ExpressionListPool,
        size,
        interpreter.Random,
        interpreter.Configuration,
        interpreter.CustomExpressions);

      interpreter.CodeStack.Push(program);
      return true;
    }
  }
}